@@ -252,22 +252,33 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
             result = await self._download_request_with_page(request, page)
         except:
             logger.exception(f"[{request.url}] Error while downloading request")
-            #return Response('', status=504)
+            return Response('', status=504)
         else:
             return result
         finally:
             logger.info(f"[{request.url}] Start finally")
             try:
-                if page is not None:
-                    if not page.is_closed():
-                        logger.info(f"[{request.url}] Close page")
-                        await page.close()
+                if page is not None and not page.is_closed():
+                    logger.info(f"[{request.url}] Close page")
+                    try:
+                        await asyncio.wait_for(page.close(), timeout=10)
+                        logger.info(f"[{request.url}] Page closed")
+                    except asyncio.TimeoutError:
+                        if self.context_semaphores['default'].locked():
+                            logger.info(f"[{request.url}] Release semaphore")
+                            self.context_semaphores['default'].release()
+                        await self._plan_user_agent_rotation()
+                    else:
+                        if self.context_semaphores['default'].locked():
+                            logger.info(f"[{request.url}] Release semaphore")
+                            self.context_semaphores['default'].release()
+                else:
+                    if self.context_semaphores['default'].locked():
+                        logger.info(f"[{request.url}] Release semaphore")
+                        self.context_semaphores['default'].release()
             except:
                 logger.error(f"[{request.url}] Error on page close")
                 raise
-            if self.context_semaphores['default'].locked():
-                logger.info(f"[{request.url}] Release semaphore")
-                self.context_semaphores['default'].release()
 
     async def _download_request_with_page(self, request: Request, page: Page) -> Response:
         start_time = time()
